package cz.bilysklep.trainings.basicjava.swingexample;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.time.LocalDate;
import java.time.Month;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingWorker;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableRowSorter;

public class JTableExample extends JFrame {

    private final PersonService personService;
    private PersonTableModel tableModel = new PersonTableModel();
    private Action loadDataAction;
    private Action removeAction;
    
    public JTableExample(PersonService personService) {
        super("JTable example");
        this.personService = personService;
        setDefaultCloseOperation(EXIT_ON_CLOSE);

        loadDataAction = new AbstractAction("Load") {
            @Override
            public void actionPerformed(ActionEvent e) {

                new SwingWorker<List<Person>, Void>() {
                    @Override
                    protected List<Person> doInBackground() throws Exception {
                        return personService.findAllPersons();
                    }

                    @Override
                    protected void done() {
                        try {
                            tableModel.setPersons(get());
                        } catch (InterruptedException ex) {
                            throw new AssertionError(ex);
                        } catch (ExecutionException ex) {
                            JOptionPane.showMessageDialog(JTableExample.this,
                                    ex.getCause().getLocalizedMessage(),
                                    "Error when loading people data", JOptionPane.ERROR_MESSAGE);
                        }

                    }
                }.execute();
            }
        };

        JToolBar toolBar = new JToolBar();
        add(toolBar, BorderLayout.NORTH);
        
        toolBar.add(loadDataAction);

        toolBar.add(new AbstractAction("Add person") {
            @Override
            public void actionPerformed(ActionEvent e) {
                Person p = new Person("Franta Vocasek",Gender.MALE,LocalDate.of(2001,Month.FEBRUARY,12));
                personService.addPerson(p);
                tableModel.addPerson(p);
            }
            
        });
        
        JTable table = new JTable(tableModel);
        // Set renderer for given column type
        table.setDefaultRenderer(Gender.class, new GenderCellRenderer());
        // Set renderer for given column
        // table.getColumnModel().getColumn(1).setCellRenderer(new GenderCellRenderer());
        table.setRowHeight(20);
        
        table.setRowSorter(new TableRowSorter<PersonTableModel>(tableModel));

        removeAction = new AbstractAction("Remove") {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (table.getSelectedRowCount() == 1) {
                    int selectedRow = table.getSelectedRow();
                    int modelRow = table.convertRowIndexToModel(selectedRow);
                    Person person = tableModel.getPerson(modelRow);
                    personService.removePerson(person);
                    tableModel.removePerson(person);
                }
            }
        };
        
        removeAction.setEnabled(false);
        toolBar.add(removeAction);

        table.getSelectionModel().addListSelectionListener(event -> {
            removeAction.setEnabled(table.getSelectedRowCount() == 1);
        });
        
        add(new JScrollPane(table));

        pack();

    }
 
    private static Icon loadIcon(String name) {
        return new ImageIcon(JTableExample.class.getResource(name));
    }
    
    private static class GenderCellRenderer extends DefaultTableCellRenderer {

        private static final I18n I18N = new I18n(GenderCellRenderer.class);
        private Map<Gender,Icon> icons = new HashMap<>();
        
        {
            icons.put(Gender.MALE, loadIcon("icons8-male-20.png"));
            icons.put(Gender.FEMALE, loadIcon("icons8-female-20.png"));
        }
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            JLabel label = (JLabel) super.getTableCellRendererComponent(
                    table, value, isSelected, hasFocus, row, column); //To change body of generated methods, choose Tools | Templates.
            label.setIcon(icons.get(value));
            label.setText(I18N.getString((Gender) value));
            //label.setHorizontalAlignment(CENTER);
            return label;
        }
        
    }
    
    public static void main(String[] args) {

        EventQueue.invokeLater(() -> {
            new JTableExample(new PersonServiceImpl()).setVisible(true);
        });

    }

}
