package cz.bilysklep.trainings.basicjava.swingexample;

import java.time.LocalDate;
import java.time.Period;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import javax.swing.table.AbstractTableModel;

public class PersonTableModel extends AbstractTableModel {

    private List<Person> persons = new ArrayList<>();
    private static final I18n I18N = new I18n(PersonTableModel.class);
    
    private static enum Column {

        NAME(Person::getName, String.class),
        GENDER(Person::getGender, Gender.class),
        BIRTHDATE(Person::getBirthDate, LocalDate.class),
        AGE(p -> Period.between(p.getBirthDate(), LocalDate.now()).getYears(), Integer.class);
        
        private Column(Function<Person, Object> getter, Class<?> columnType) {
            this.getter = getter;
            this.columnType = columnType;
        }
        
        private final Function<Person,Object> getter;
        private final Class<?> columnType;
        
    }
    
    public void setPersons(List<Person> persons) {
        this.persons = new ArrayList<>(persons);
        fireTableDataChanged();
    }

    public Person getPerson(int rowIndex) {
        return persons.get(rowIndex);
    }

    public void removePerson(Person person) {
        int index = persons.indexOf(person);
        persons.remove(index);
        fireTableRowsDeleted(index, index);
    }
    
    public void addPerson(Person person) {
        int index = persons.size();
        persons.add(person);
        fireTableRowsInserted(index, index);
    }
    
    @Override
    public int getRowCount() {
        return persons.size();
    }

    @Override
    public int getColumnCount() {
        return Column.values().length;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Person person = getPerson(rowIndex);
        return Column.values()[columnIndex].getter.apply(person);
    }

    @Override
    public String getColumnName(int columnIndex) {
        //return Column.values()[columnIndex].toString();
        return I18N.getString(Column.values()[columnIndex]);
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return Column.values()[columnIndex].columnType;
    }

    
    
}
